﻿using System;
using System.Net;
using System.Windows;
using System.ComponentModel;
using System.Collections.Generic;
using System.Threading;
using System.IO;

namespace SampleCode
{
    public class QueueProcessor<T>
    {
        private BackgroundWorker backgroundWorker;
        private readonly IWorker worker;

        public QueueProcessor(ObservableQueue<T> queueToMonitor, IWorker worker)
        {
            ((SampleCode.App)App.Current).ConnectionStatusChangedEvent += new ConnectionStatusChangedEventHandler(QueueProcessor_ConnectionStatusChangedEvent);
            queueToMonitor.ItemAddedEvent += new
                ItemAddedEventHandler(PendingData_ItemAddedEvent);
            this.worker = worker;
            SetUpBackgroundWorker();
            if ((((SampleCode.App)App.Current).IsConnected) && (!backgroundWorker.IsBusy)
                && (((SampleCode.App)App.Current).PendingData.Count > 0))
            {
                backgroundWorker.RunWorkerAsync();
            }
        }

        private void PendingData_ItemAddedEvent()
        {
            if ((((SampleCode.App)App.Current).IsConnected) && (!backgroundWorker.IsBusy))
            {
                backgroundWorker.RunWorkerAsync();
            }
        }

        private void SetUpBackgroundWorker()
        {
            backgroundWorker = new BackgroundWorker();
            backgroundWorker.WorkerSupportsCancellation = true;
            backgroundWorker.WorkerReportsProgress = true;
            backgroundWorker.DoWork += new DoWorkEventHandler(this.worker.Execute);
            backgroundWorker.ProgressChanged += new
                ProgressChangedEventHandler(backgroundWorker_ProgressChanged);
            backgroundWorker.RunWorkerCompleted += new
                RunWorkerCompletedEventHandler(backgroundWorker_RunWorkerCompleted);
        }

        private void backgroundWorker_RunWorkerCompleted(object sender,
            RunWorkerCompletedEventArgs e)
        {
            if (e.Cancelled)
            {
                // Handle cancellation
            }
            else if (e.Error != null)
            {
                // Handle error
            }
            else
            {
                // Handle completion if necessary
            }
        }

        private void backgroundWorker_ProgressChanged(object sender,
            ProgressChangedEventArgs e)
        {
            // Raise event to notify observers.
        }

        private void QueueProcessor_ConnectionStatusChangedEvent(bool isConnected)
        {
            if (isConnected)
            {
                if (!backgroundWorker.IsBusy)
                {
                    backgroundWorker.RunWorkerAsync();
                }
            }
            else
            {
                backgroundWorker.CancelAsync();
            }
        }
    }

}
