﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;
using System.Threading;
using System.Collections.ObjectModel;
using System.Windows.Threading;

namespace SampleCode
{
    public class Repository
    {
        private ObservableCollection<NewsItem> news = new ObservableCollection<NewsItem>();
        private DispatcherTimer timer = new DispatcherTimer();
        private const int TIMER_INTERVAL = 1;

        public Repository()
        {
            ((SampleCode.App)App.Current).ConnectionStatusChangedEvent += new ConnectionStatusChangedEventHandler(Repository_ConnectionStatusChangedEvent);
            if (((SampleCode.App)App.Current).IsConnected)
            {
                RetrieveNews();
                StartTimer();
            }
        }

        private void Repository_ConnectionStatusChangedEvent(bool isConnected)
        {
            if (isConnected)
            {
                StartTimer();
            }
            else
            {
                StopTimer();
            }
        }

        private void StopTimer()
        {
            this.timer.Stop();
        }

        private void StartTimer()
        {
            this.timer.Interval = TimeSpan.FromMinutes(1);
            this.timer.Tick += new EventHandler(timer_Tick);
            this.timer.Start();
        }

        void  timer_Tick(object sender, EventArgs e)
        {
         	if (((SampleCode.App)App.Current).IsConnected)
            {
                RetrieveNews();
            }
        }

        private void RetrieveNews()
        {
            //Get latest news from server
            List<NewsItem> list = GetNewsFromServer();
            if (list.Count > 0)
            {
                lock (this.news)
                {
                    foreach (NewsItem item in list)
                    {
                        this.news.Add(item);
                    }
                }
            }
        }

        private List<NewsItem> GetNewsFromServer()
        {
            //Simulate retrieval from server
            List<NewsItem> list = new List<NewsItem>();
            for (int i = 0; i < 5; i++)
            {
                NewsItem newsItem = new NewsItem() 
                    { Headline = "Something happened at " + DateTime.Now.ToLongTimeString(),
                      Body = "On " + DateTime.Now.ToLongDateString() + " something happened.  We'll know more later." };
                list.Add(newsItem);
            }
            return list;
        }

        public ObservableCollection<NewsItem> News
        {
            get
            {
                return this.news;
            }
            set
            {
                this.news = value;
            }
        }

    }
}
